LoadingView = Backbone.View.extend({
    el: "#content-loading",

    initialize: function() {
        $(this.el).hide();
    },
});

ContentView = Backbone.View.extend({
    el: "#content",

    initialize: function() {
        this.render();
        this.libraryView = new LibraryView();
        this.sidebarView = new SidebarView();
        this.playerView = new PlayerView();
    },

    render: function() {
        $(this.el).hide();
    }
});

PlaylistTabView = Backbone.View.extend({
    el: "#current_playlist",

    initialize: function() {
        this.render();
    },

    render: function() {
    }
});

LibraryView = Backbone.View.extend({
    el: "#search_library",

    initialize: function() {
        //Initialize new Track, Album, and Artist Views
        //and render them
        this.render();
        this.tracksView = new EntryCollectionView({el:"#tracks", collection:app.trackCollection});
        this.albumsView = new AlbumCollectionView({el: "#albums", collection:app.albumCollection});
        this.artistsView = new ArtistCollectionView({el: "#artists", collection:app.artistCollection});
    },

    render: function() {
    }
});

//Renders individual artists
ArtistCollectionView = Backbone.View.extend({
    initialize: function() {
        this.collection.on("add", this.addAll, this);
        this.collection.on("reset", this.addAll, this);
    },

    addItem: function(item) {
        view = new ArtistView({model: item});
        this.$el.append(view.render().el);
    },

    addAll: function(item) {
        $(this.el).html('');
        if (window.filter !== undefined)
            _.each(this.collection.match(), this.addItem, this);
        else
            this.collection.each(this.addItem, this);
    }
});

ArtistView = Backbone.View.extend({
    tagName: "li",
    className: "artist",
    template: _.template($("#artist-template").html()),

    events: {
        'click .icon-plus': 'addToPlaylist',
        'click .title': 'addToPlaylistAndPlay'
    },

    render: function() {
        $(this.el).html(this.template(this.model.toJSON()));
        return this;
    },

    addToPlaylist: function() {
        albums = this.model.get("albums");

        for (var i = 0; i < albums.length; i++) {
            tracks = albums[i].get("tracks");

            for (var j = 0; j < tracks.length; j++) {
                app.playlistCollection.add(tracks[j]);        
            }
        }
        
        app.playlistView.render();
    },

    addToPlaylistAndPlay: function() {
        //Play the first item in this collection
        firstTrack = this.model.get("albums")[0].get("tracks")[0] 
        app.getPlayer().play(firstTrack);

        //Add to playlist after for highlighting
        this.addToPlaylist();
        app.getInfoView().render(firstTrack);
    },
});

//Renders individual albums
AlbumCollectionView = Backbone.View.extend({
    initialize: function() {
        this.collection.on("add", this.addAll, this);
        this.collection.on("reset", this.addAll, this);
    },

    addItem: function(item) {
        view = new AlbumView({model: item});
        this.$el.append(view.render().el);
    },

    addAll: function() {
        $(this.el).html('');
        if (window.filter !== undefined)
            _.each(this.collection.match(), this.addItem, this);
        else
            this.collection.each(this.addItem, this);
    }
});

AlbumView = Backbone.View.extend({
    tagName: "li",
    className: "album",
    template: _.template($("#album-template").html()),

    events: {
        'click .icon-plus': 'addToPlaylist',
        'click .title': 'addToPlaylistAndPlay'
    },

    render: function() {
        $(this.el).html(this.template(this.model.toJSON()));
        return this;
    },

    addToPlaylist: function() {
        trackModels = this.model.get("tracks")

        for (var i = 0; i < trackModels.length; i++) {
            app.playlistCollection.add(trackModels[i]);        
        }

        //Take care of highlighting
        app.playlistView.render();
    },

    addToPlaylistAndPlay: function() {
        //Play the first one in this collection
        firstTrack = this.model.get("tracks")[0]
        app.getPlayer().play(firstTrack);

        //Add to playlist after to make highlighting
        //correct
        this.addToPlaylist();
        app.getInfoView().render(firstTrack);
    },
});


//Renders individual tracks
EntryCollectionView = Backbone.View.extend({
    initialize: function() {
        this.collection.on("add", this.addAll, this);
        this.collection.on("reset", this.addAll, this);
    },

    addItem: function(item) {
        view = new EntryView({model: item});
        this.$el.append(view.render().el);
    },

    addAll: function() {
        $(this.el).html(''); 
        if (window.filter !== undefined)
            _.each(this.collection.match(), this.addItem, this);
        else
            this.collection.each(this.addItem, this);
    },
});

EntryView = Backbone.View.extend({
    tagName: "li",
    className: "entry",
    template: _.template($("#entry-template").html()),

    initialize: function() {
    },

    events: {
        'click .icon-plus': 'addToPlaylist',
        'click .title': 'addToPlaylistAndPlay'
    },

    render: function() {
        $(this.el).html(this.template(this.model.toJSON()));
        return this;
    },

    addToPlaylist: function() {
        app.playlistCollection.add(this.model);
        app.playlistView.render();
    },

    addToPlaylistAndPlay: function() {
        app.getPlayer().play(this.model);

        //Have to do this second so that it will
        //trigger highlighing
        this.addToPlaylist();
        app.getInfoView().render(this.model);
    },
});

PlaylistCollectionView = Backbone.View.extend({
    el: "#playlist",

    initialize: function() {
        this.collection.on("add", this.addAll, this);
        this.collection.on("reset", this.addAll, this);
    },

    render: function() {
        this.addAll();
    },

    addItem: function(item) {
        var view = new PlaylistView({model: item});
        this.$el.append(view.render().el);

        //Compare by id for right now, this will likely change
        //in the future once you can add songs with the same id
        //to the playlist
        if (item.get("id") === app.getPlayer().nowPlaying.get("id")) {
            view.highlight();
        }
    },

    addAll: function() {
        this.$el.html('');
        this.collection.each(this.addItem, this);
    },

    clear: function() {
        this.$el.html('');
        this.collection.reset();
        app.getPlayer().pause();
    }
});

PlaylistView = Backbone.View.extend({
    tagName: "li",
    className: "playlistEntry",
    template: _.template($("#playlist-item-template").html()),

    initialize: function() {
        this.render();
    },

    events: {
        "click": "highlight",
        "click .play": "playItem",
        "click .icon-remove": "removeItem"
    },

    render: function() {
        $(this.el).html(this.template(this.model.toJSON()));
        return this;
    },

    highlight: function() {
        $(this.el).addClass("highlight");
        $(this.el).siblings().removeClass("highlight");
    },

    playItem: function() {
        app.getPlayer().play(this.model);
        app.getInfoView().render(this.model);
    },

    removeItem: function() {
        //Logic for removing from player
        app.getPlayer().remove(this.model);

        //We have to remove from collection second because
        //we still want the item to be in the collection so
        //we know what item to play next
        app.playlistCollection.remove(this.model);
        $(this.el).html('');
    }
});


PlaylistControlView = Backbone.View.extend({
    el: "#playlist-controls",
    templateAutoloopOn: _.template($("#autoloop-on-template").html()),
    templateAutoloopOff: _.template($("#autoloop-off-template").html()),

    initialize: function() {
        this.autoloopOn = false;     
        this.renderOn();
    },

    events: {
        'click #autoloop-status': 'toggleRender',
        'click #clear-playlist': 'clearPlaylist',
        'click #shuffle-playlist': 'shufflePlaylist'
    },

    renderOn: function() {
        $("#autoloop-status").html(this.templateAutoloopOn());
    },

    renderOff: function() {
        $("#autoloop-status").html(this.templateAutoloopOff());
    },

    toggleRender: function() {
        this.autoloopOn = !this.autoloopOn;

        //If it's on already, we want the
        //option to turn if off, otherwise we
        //want to turn it off
        if (this.autoloopOn)
            this.renderOff();
        else
            this.renderOn();
    },

    clearPlaylist: function() {
        app.playlistView.clear();
    },

    shufflePlaylist: function() {
        app.playlistCollection.reset(_.shuffle(app.playlistCollection.toJSON()));
    }
});

SidebarView = Backbone.View.extend({
    el: "#sidebar",

    initialize: function() {
        //this.input = this.$(".input-large");
        this.time = 200;
    },

    events: {
        "keyup .input-large": "startTimer",
        "keydown .input-large": "clearTimer",
        "focus .input-large": "focusLibrary"
    },

    startTimer: function(e) {
        window.typingTimer = setTimeout(this.filter, this.time);
    },

    clearTimer: function(e) {
        clearTimeout(window.typingTimer);
    },

    filter: function(e) {
        window.filter = $("#query").val().toLowerCase();
        app.trackCollection.trigger('reset');
        app.albumCollection.trigger('reset');
        app.artistCollection.trigger('reset');
    },

    focusLibrary: function() {
        $("#navigation a:first").tab('show'); 
    }
});

InfoView = Backbone.View.extend({
    el: "#info",
    template: _.template($("#info-template").html()),
    
    initialize: function() {
    },

    render: function(model) {
        $(this.el).html(this.template(model.toJSON()));
    },

    clear: function() {
        $(this.el).html("");
    }
});

PlayerView = Backbone.View.extend({
    el: "#player",
    nowPlaying: undefined,
    
    initialize: function() {
        a = audiojs.createAll({
            trackEnded: this.playNext
        });

        this.audio = a[0];
    },

    //Plays a model
    play: function(model) {
        this.audio.load("/item/" + model.id + "/file");
        this.audio.play();
        this.nowPlaying = model;
    },

    playNext: function() {
        //Had to do this because .indexOf fails
        currentIndex = app.indexById(app.playlistCollection, app.getPlayer().nowPlaying.get("id"))

        if (currentIndex < 0) {
            //This means that the current playing item wasn't in the playlist
            //collection at all.  it should NEVER happen.
            console.log("ERROR: Current item NOT IN playlist");
        }

        //This is the next index
        nextIndex = currentIndex + 1

        if (nextIndex >= app.playlistCollection.length) {
            if (app.playlistControlView.autoloopOn) {
                nextIndex = 0;
            }
            else {
                //If we have nothing left to do, pause it
                app.getPlayer().pause();
                app.getInfoView().clear();
                return;
            }
        }
        
        nextTrack = app.playlistCollection.at(nextIndex);

        //console.log("Next playing is: ");
        //console.log(nextTrack.toJSON());

        //Play and render
        app.getPlayer().play(nextTrack);
        app.getInfoView().render(nextTrack);

        //Advance highlighting
        app.playlistView.render();
    },

    //We have to have this function because if we
    //want to remove something and it is currently
    //playing we want to go to the next item
    remove: function(model) {
        if (this.nowPlaying != null && this.nowPlaying.id === model.id)
            app.getPlayer().playNext();
    },

    pause: function() {
        this.audio.pause();
    }
});

EntryModel = Backbone.Model.extend({
   defaults: {
       title: "Track Title",
       album: "Album Title",
       artist: "Artist Name",
       album_id: 45,
       length: 0,
       track: 0,
       tracktotal: 0,
       mb_trackid: 0,
       mb_albumid: 0,
       mb_artistid: 0
   }
});

EntryCollection = Backbone.Collection.extend({
    model: EntryModel,
    match: function() {
        if (window.filter != undefined) {
            return this.filter(function(item) {
                return item.get("title").toLowerCase().indexOf(window.filter) > -1;
            });
        }
    }
});

AlbumModel = Backbone.Model.extend({
   defaults: {
       album: "Album Title",
       artist: "Artist Name",
       tracktotal: 0,
       tracks: []
   }
});

AlbumCollection = Backbone.Collection.extend({
    model: AlbumModel,
    match: function() {
        if (window.filter != undefined) {
            return this.filter(function(item) {
                return item.get("album").toLowerCase().indexOf(window.filter) > -1;
            });
        }
    }
});

ArtistModel = Backbone.Model.extend({
   defaults: {
       artist: "Artist Title",
       albums: []
   }
});

ArtistCollection = Backbone.Collection.extend({
    model: ArtistModel,
    match: function() {
        if (window.filter != undefined) {
            return this.filter(function(item) {
                return item.get("artist").toLowerCase().indexOf(window.filter) > -1;
            });
        }
    }
});

PlaylistCollection = Backbone.Collection.extend({
    model: EntryModel,
});

var app = {
    initialize: function() {
        app.trackCollection = new EntryCollection();
        app.albumCollection = new AlbumCollection();
        app.artistCollection = new ArtistCollection();

        app.views = {
            loadingView: new LoadingView(),
            contentView: new ContentView()
        }

        app.playlistCollection = new PlaylistCollection();
        app.playlistView = new PlaylistCollectionView({collection: app.playlistCollection});

        //Initialize playlist drag and drop sorting
        $("#playlist").sortable({
            update: function(event, ui) {
                models = []
                $("#playlist li").each(function(index) {
                    id = $(this).attr("data-id");
                    if (typeof id !== "undefined" && id !== false) {
                        //Get the associated model
                        models.push(app.trackCollection.get(id).toJSON());
                    }
                });
                //This is kind of a pain because it will cause a re-render
                //but it makes sure everything is in sync, so it works out
                app.playlistCollection.reset(models) 
            }
        });

        app.infoView = new InfoView();
    },

    getPlayer: function() {
        return app.views.contentView.playerView;
    },

    getPlaylist: function() {
        return app.playlistCollection;
    },

    getInfoView: function() {
        return app.infoView;
    },

    showView: function(view) {
        if (app.views.current != undefined) {
            $(app.views.current.el).hide();
        }

        app.views.current = view;
        $(app.views.current.el).show();
    },

    indexById: function(collection, id) {
        for (var i = 0; i < collection.length; i++) {
            if (id === collection.at(i).get("id"))
                return i;
        }
        return -1;
    }
}

app.initialize()
app.showView(app.views.loadingView);

//Initialize Playlist Controls
app.playlistControlView = new PlaylistControlView();

//Global objects to be used in the countdown latch
//callback and construction of objects
window.tracks = undefined;
window.albums = undefined;
window.artists = undefined;

//Provide a countdown latch for syncronizing the
//callbacks so we don't have to do that in the getJSON
//callbacks
CountdownLatch = function() {
    this.initialize = function(n, callback) {
        this.n = n;
        this.callback = callback;
    }

    this.countDown = function() {
        this.n--;
        if (this.n === 0)
            this.callback();
    }
}

c = new CountdownLatch()
c.initialize(3, function() {
    //Go through each album in the data and
    //construct the object from item models
    for (var i = 0; i < window.albums.length; i++) {

        // Store the tracks temporarily
        temp = albums[i].tracks.slice();
        window.albums[i].tracks = []

        for (var j = 0; j < temp.length; j++) {
            window.albums[i].tracks.push(
                app.trackCollection.get(temp[j])
            );
        }

        //Sort based on track number
        window.albums[i].tracks = _.sortBy(window.albums[i].tracks, function(m) {
           return m.get("track");
        });

    }

    app.albumCollection.reset(window.albums);
    
    //Go through each artist in the data and construct
    //the object from album models
    for (var i = 0; i < window.artists.length; i++) {
        temp = window.artists[i].albums.slice();
        window.artists[i].albums = [];

        for (var j = 0; j < temp.length; j++) {
            window.artists[i].albums.push(
                app.albumCollection.get(temp[j])
            );
        }
    }

    app.artistCollection.reset(window.artists);

    app.showView(app.views.contentView);
    $("#query").focus();
});

//Actually do the requests here
$.getJSON('/items', function(data) {
    app.trackCollection.reset(data.items);
}).complete(function() {
    c.countDown();
});

$.getJSON('/albums', function(data) {
    window.albums = data.albums;
}).complete(function() {
    c.countDown();
});

$.getJSON('/artists', function(data) {
    window.artists = data.artists;
}).complete(function() {
    c.countDown();
});

